#include "scanner.h"
#include "config.h"
#include "utils.h"
#include "ui.h"
#include <vector>
#include <U8g2lib.h>

// --- Global Variable Definitions ---
AccessPointInfo ap_list[MAX_APS];
uint8_t ap_count = 0;
StationInfo sta_list[MAX_STAS];
uint8_t sta_count = 0;
uint8_t current_channel = DEFAULT_WIFI_CHANNEL;
bool scanner_active = false;
static unsigned long last_channel_hop_time = 0;
static int selected_list_index = 0;
static int display_offset = 0;

// --- External UI Elements ---
extern U8G2_SSD1306_128X64_NONAME_F_HW_I2C oled;
extern bool needsRedraw;

// --- Initialization ---
void scanner_init() {
    Serial.println("Scanner Initialized.");
    ap_count = 0;
    sta_count = 0;
    scanner_active = false;
    selected_list_index = 0;
    display_offset = 0;
    needsRedraw = true;
}

// --- Stop Scanner ---
void scanner_stop() {
    if (scanner_active) {
        Serial.println("Stopping Scanner...");
        wifi_promiscuous_enable(0);
        scanner_active = false;
        Serial.println("Scanner Stopped.");
        needsRedraw = true;
    } else {
        Serial.println("Scanner already stopped.");
    }
}

// --- Main Update Loop ---
void scanner_update() {
    if (!scanner_active) {
         Serial.println("Scanner Update: Enabling promiscuous mode.");
         wifi_set_opmode(STATION_MODE);
         wifi_set_promiscuous_rx_cb(packet_capture_callback);
         wifi_promiscuous_enable(1);
         scanner_set_channel(current_channel);
         scanner_active = true;
         last_channel_hop_time = millis();
    }

    unsigned long current_time = millis();

    if (current_time - last_channel_hop_time > SCAN_CHANNEL_HOP_INTERVAL) {
        current_channel++;
        if (current_channel > 13) {
            current_channel = 1;
        }
        scanner_set_channel(current_channel);
        last_channel_hop_time = current_time;
        needsRedraw = true;
    }
}

// --- Set WiFi Channel ---
void scanner_set_channel(uint8_t channel) {
    if (channel < 1 || channel > 14) {
        Serial.printf("Invalid channel: %d\n", channel);
        return;
    }
    current_channel = channel;
    wifi_set_channel(current_channel);
}

// --- Packet Capture Callback (Called by SDK) ---
void packet_capture_callback(uint8_t *buf, uint16_t len) {
    if (!scanner_active || buf == nullptr || len <= sizeof(wifi_promiscuous_pkt_t)) {
        return;
    }

    wifi_promiscuous_pkt_t *packet = (wifi_promiscuous_pkt_t*)buf;
    process_wifi_packet(packet->payload, len - sizeof(wifi_pkt_rx_ctrl_t));
}

// --- Process Captured WiFi Packet ---
void process_wifi_packet(uint8_t *payload, uint16_t payload_len) {
    if (payload_len < sizeof(mac_header_t)) {
         return;
    }

    mac_header_t *header = (mac_header_t*)payload;
    wifi_promiscuous_pkt_t *packet = nullptr;
    if (payload >= (uint8_t*)sizeof(wifi_pkt_rx_ctrl_t)) {
         packet = (wifi_promiscuous_pkt_t*)(payload - sizeof(wifi_pkt_rx_ctrl_t));
    } else {
        Serial.println("ERROR: Cannot calculate packet pointer!");
        return;
    }

    if (packet == nullptr) {
         Serial.println("ERROR: Invalid packet pointer calculated!");
         return;
    }

    int8_t rssi = packet->rx_ctrl.rssi;
    uint8_t channel = packet->rx_ctrl.channel;

    uint8_t type = header->frame_control.type;
    uint8_t subtype = header->frame_control.subtype;

    // Identify APs (Beacons and Probe Responses)
    if (type == 0 && (subtype == 0x08 || subtype == 0x05)) {
        uint16_t expected_min_len = sizeof(mac_header_t) + sizeof(beacon_probe_fixed_params_t);
        if (payload_len >= expected_min_len) {
            beacon_probe_fixed_params_t *params = (beacon_probe_fixed_params_t*)(payload + sizeof(mac_header_t));
            uint8_t* tagged_params_start = payload + sizeof(mac_header_t) + sizeof(beacon_probe_fixed_params_t);
            uint16_t tagged_params_len = payload_len - expected_min_len;
            add_or_update_ap(header, params, rssi, channel, tagged_params_start, tagged_params_len);
            needsRedraw = true;
        } else {
        }
    }
    // Identify Stations (From various frame types)
    else {
        if (header->addr2[0] != 0xFF && header->addr2[0] != 0x01) {
             uint8_t* bssid = nullptr;
             if (header->frame_control.flags & 0x01) {
                 bssid = header->addr1;
             } else if (header->frame_control.flags & 0x02) {
                 bssid = header->addr3;
             } else { 
                 if (header->addr3[0] != 0xFF && header->addr3[0] != 0x01) {
                    bssid = header->addr3;
                 }
             }
             if (bssid && bssid[0] != 0xFF && bssid[0] != 0x01 && bssid[0] != 0x00) {
                add_or_update_sta(header->addr2, bssid, rssi);
                needsRedraw = true;
             }
        }
    }
}

// --- Find Existing AP by BSSID ---
int find_ap(const uint8_t* bssid) {
    for (int i = 0; i < ap_count; ++i) {
        if (memcmp(ap_list[i].bssid, bssid, 6) == 0) {
            return i;
        }
    }
    return -1;
}

// --- Find Existing Station by MAC ---
int find_sta(const uint8_t* mac) {
    for (int i = 0; i < sta_count; ++i) {
        if (memcmp(sta_list[i].mac, mac, 6) == 0) {
            return i;
        }
    }
    return -1;
}

// --- Add or Update Access Point Info ---
void add_or_update_ap(const mac_header_t* header, const beacon_probe_fixed_params_t* params, int8_t rssi, uint8_t channel, const uint8_t* tagged_params, uint16_t tagged_params_len) {
    const uint8_t* bssid = header->addr3;
    int index = find_ap(bssid);

    if (index == -1) {
        if (ap_count < MAX_APS) {
            index = ap_count;
            memcpy(ap_list[index].bssid, bssid, 6);
            ap_list[index].ssid[0] = '\0';
            ap_list[index].packet_count = 0;
            ap_count++;
            Serial.printf("New AP: %s (RSSI: %d, Ch: %d)\n", macToString(bssid).c_str(), rssi, channel);
        } else {
            return;
        }
    } else {
    }

    ap_list[index].rssi = rssi;
    ap_list[index].channel = channel;
    ap_list[index].last_seen = millis();
    ap_list[index].packet_count++;
    ap_list[index].selected = false;

    // Parse Tagged Parameters for SSID
    uint16_t pos = 0;
    bool ssid_found = false;

    while (pos + 1 < tagged_params_len) {
        uint8_t tag_num = tagged_params[pos];
        uint8_t tag_len = tagged_params[pos + 1];

        if (pos + 2 + tag_len > tagged_params_len) {
            break;
        }

        if (tag_num == 0) {
            ssid_found = true;
            if (tag_len == 0) {
                ap_list[index].ssid[0] = '\0';
            } else {
                uint16_t copy_len = (tag_len >= MAX_SSID_LEN) ? (MAX_SSID_LEN - 1) : tag_len;
                if (pos + 2 + copy_len <= tagged_params_len) {
                     memcpy(ap_list[index].ssid, tagged_params + pos + 2, copy_len);
                     ap_list[index].ssid[copy_len] = '\0';
                } else {
                    ap_list[index].ssid[0] = '\0';
                }
            }
        }

        pos += 2 + tag_len;
    }

    if (!ssid_found) {
        ap_list[index].ssid[0] = '\0';
    }

    ap_list[index].encryption = get_encryption_type(params->capability_info, tagged_params, tagged_params_len);
}

// --- Add or Update Station Info ---
void add_or_update_sta(const uint8_t* mac, const uint8_t* bssid, int8_t rssi) {
    int index = find_sta(mac);

    if (index == -1) {
        if (sta_count < MAX_STAS) {
            index = sta_count;
            memcpy(sta_list[index].mac, mac, 6);
            sta_list[index].packet_count = 0;
            sta_count++;
             Serial.printf("New STA: %s (AP: %s)\n", macToString(mac).c_str(), macToString(bssid).c_str());
        } else {
            return;
        }
    }

    sta_list[index].rssi = rssi;
    sta_list[index].last_seen = millis();
    sta_list[index].packet_count++;
    sta_list[index].selected = false;
    memcpy(sta_list[index].bssid_ap, bssid, 6);
}


// --- Determine Encryption Type ---
EncryptionType get_encryption_type(const uint16_t capabilities, const uint8_t* tagged_params, uint16_t tagged_params_len) {
    bool privacy_bit = capabilities & 0x0010;

    if (!privacy_bit) {
        return EncryptionType::OPEN;
    }

    bool found_wpa = false;
    bool found_rsn = false;

    uint16_t pos = 0;
    while (pos + 1 < tagged_params_len) {
        uint8_t tag_num = tagged_params[pos];
        uint8_t tag_len = tagged_params[pos + 1];

        if (pos + 2 + tag_len > tagged_params_len) {
             break;
        }

        if (tag_num == 0xDD && tag_len >= 4) {
             if (tagged_params[pos + 2] == 0x00 && tagged_params[pos + 3] == 0x50 &&
                 tagged_params[pos + 4] == 0xF2 && tagged_params[pos + 5] == 0x01) {
                 found_wpa = true;
             }
        }
        else if (tag_num == 0x30 && tag_len >= 2) {
            found_rsn = true;
        }

        pos += 2 + tag_len;
    }

    if (found_rsn && found_wpa) {
        return EncryptionType::WPA_WPA2_PSK;
    }
    if (found_rsn) {
         return EncryptionType::WPA2_PSK;
    }
    if (found_wpa) {
         return EncryptionType::WPA_PSK;
    }
    return EncryptionType::WEP;
}

// --- Convert Encryption Enum to String ---
String encryption_to_string(EncryptionType enc) {
    switch (enc) {
        case EncryptionType::OPEN: return "Open";
        case EncryptionType::WEP: return "WEP";
        case EncryptionType::WPA_PSK: return "WPA";
        case EncryptionType::WPA2_PSK: return "WPA2";
        case EncryptionType::WPA_WPA2_PSK: return "WPA/2";
        case EncryptionType::WPA2_ENTERPRISE: return "WPA2-E";
        default: return "Unkn";
    }
}

// --- Draw Scanner UI ---
void scanner_draw() {
    oled.setFont(u8g2_font_5x7_tf);

    char buffer[40];
    snprintf(buffer, sizeof(buffer), "Ch:%d APs:%d STAs:%d", current_channel, ap_count, sta_count);
    oled.drawStr(0, 23, buffer);

    oled.drawHLine(0, 25, SCREEN_WIDTH);

    int yPos = 33;
    int line_height = 8;
    int total_items = ap_count + sta_count;

    const int lines_per_item = 2;
    const int gap_between_items = 1;
    const int item_block_height = (line_height * lines_per_item) + gap_between_items;
    const int items_to_display = 2;

    for (int i = 0; i < items_to_display; ++i) {
        int current_item_index = display_offset + i;
        if (current_item_index >= total_items) break;

        bool is_selected = (current_item_index == selected_list_index);
        const char* select_char = is_selected ? ">" : " ";

        if (current_item_index < ap_count) {
            AccessPointInfo* ap = &ap_list[current_item_index];
            ap->selected = is_selected;

            String encStr = encryption_to_string(ap->encryption);
            snprintf(buffer, sizeof(buffer), "%s%-16.16s %s", select_char, ap->ssid[0] ? ap->ssid : "[Hidden]", encStr.c_str());
            oled.drawStr(0, yPos, buffer);

            snprintf(buffer, sizeof(buffer), " %s R:%d", macToString(ap->bssid).c_str(), ap->rssi);
            oled.drawStr(0, yPos + line_height, buffer);

        } else {
            int sta_index = current_item_index - ap_count;
            StationInfo* sta = &sta_list[sta_index];
            sta->selected = is_selected;

            snprintf(buffer, sizeof(buffer), "%s%s R:%d", select_char, macToString(sta->mac).c_str(), sta->rssi);
            oled.drawStr(0, yPos, buffer);

            if (sta->bssid_ap[0] != 0 || sta->bssid_ap[1] != 0) {
                 snprintf(buffer, sizeof(buffer), " AP: %s", macToString(sta->bssid_ap).c_str());
                 oled.drawStr(0, yPos + line_height, buffer);
            } else {
                 oled.drawStr(0, yPos + line_height, " AP: [Scanning...]");
            }
        }
        yPos += item_block_height;
    }

    if (total_items > items_to_display) {
        if (display_offset > 0) {
             oled.drawTriangle(SCREEN_WIDTH - 6, 27, SCREEN_WIDTH - 9, 31, SCREEN_WIDTH - 3, 31);
        }
        if (display_offset + items_to_display < total_items) {
             oled.drawTriangle(SCREEN_WIDTH - 6, SCREEN_HEIGHT - 1, SCREEN_WIDTH - 9, SCREEN_HEIGHT - 5, SCREEN_WIDTH - 3, SCREEN_HEIGHT - 5);
        }
    }
}

// --- Handle Button Input for Scanner Mode ---
void scanner_handle_input(ButtonPressType pressType) {
    if (!scanner_active) return;

    if (pressType != ButtonPressType::NO_PRESS) {
    }

    int total_items = ap_count + sta_count;
    if (total_items == 0 && pressType != ButtonPressType::LONG_PRESS) return;

    const int items_per_page = 2;

    if (pressType == ButtonPressType::SHORT_PRESS) {
        selected_list_index = (selected_list_index + 1) % total_items;

        if (selected_list_index < display_offset) {
            display_offset = selected_list_index;
        } else if (selected_list_index >= display_offset + items_per_page) {
            display_offset = selected_list_index - items_per_page + 1;
             if (display_offset > total_items - items_per_page) {
                 display_offset = total_items - items_per_page;
             }
        }
         if (display_offset < 0) display_offset = 0;

        needsRedraw = true;
    } else if (pressType == ButtonPressType::LONG_PRESS) {
         scanner_stop();
         currentState = UIState::MAIN_MENU;
         currentMenuIndex = 0;
         needsRedraw = true;
    }
} 