#ifndef SCANNER_H
#define SCANNER_H

#include <Arduino.h>
#include <ESP8266WiFi.h> // For WiFi functions and types
#include "config.h"      // Include config for other potential settings
#include "ui.h"          // For ButtonPressType
#include <vector>

// --- Constants ---
#define MAX_APS 50 // Maximum number of APs to store
#define MAX_STAS 30 // Maximum number of STAs to store
#define MAX_SSID_LEN 32
#define SCAN_CHANNEL_HOP_INTERVAL 500 // ms
#define MAX_DISPLAY_LINES 4 // Number of APs/STAs to show on screen at once
#define SCAN_CHANNEL_TIME 150 // ms per channel
#define SCAN_PROBE_DELAY 10   // ms between probe requests

// --- WiFi Packet Structures (Simplified) ---
// Structure for the basic 802.11 frame control field
typedef struct {
    uint8_t protocol_version : 2;
    uint8_t type : 2;
    uint8_t subtype : 4;
    uint8_t flags;
} __attribute__((packed)) frame_control_t;

// Structure for the standard 802.11 MAC header (adjust based on frame type)
typedef struct {
    frame_control_t frame_control;
    uint16_t duration_id;
    uint8_t addr1[6]; // Receiver MAC
    uint8_t addr2[6]; // Transmitter MAC
    uint8_t addr3[6]; // BSSID or Source/Dest MAC depending on frame type
    uint16_t seq_ctrl;
    // Optional fields like addr4, QoS control, HT control follow
} __attribute__((packed)) mac_header_t;

// Structure for Beacon/Probe Response fixed parameters
typedef struct {
    uint64_t timestamp;
    uint16_t beacon_interval;
    uint16_t capability_info;
    // Tagged parameters follow (SSID, Rates, Channel, etc.)
} __attribute__((packed)) beacon_probe_fixed_params_t;

// Structure to hold SDK packet info
typedef struct {
    signed rssi : 8;            // signal intensity of packet
    unsigned rate : 4;          // data rate
    unsigned is_group : 1;
    unsigned : 1;               // reserve
    unsigned sig_mode : 2;      // 0:is not 11n packet; 1:is 11n packet
    unsigned legacy_length : 12; // if not 11n packet, shows length of packet.
    unsigned damatch0 : 1;
    unsigned damatch1 : 1;
    unsigned bssidmatch0 : 1;
    unsigned bssidmatch1 : 1;
    unsigned mcs : 7;           // if is 11n packet, shows the modulation and code scheme index
    unsigned cwb : 1;           // if is 11n packet, shows the channel bandwidth
    unsigned ht_length : 16;    // if is 11n packet, shows length
    unsigned smoothing : 1;
    unsigned not_sounding : 1;
    unsigned : 1;               // reserve
    unsigned aggregation : 1;   // aggregation
    unsigned stbc : 2;          // space time block code
    unsigned fec_coding : 1;    // forward error correction code
    unsigned sgi : 1;           // short guide interval
    unsigned rxend_state : 8;
    unsigned ampdu_cnt : 8;
    unsigned channel : 4;       // channel number
    unsigned : 12;              // reserve
} wifi_pkt_rx_ctrl_t;

typedef struct {
    wifi_pkt_rx_ctrl_t rx_ctrl;
    uint8_t payload[0]; // Flexible array member for packet data
} wifi_promiscuous_pkt_t;


// --- Data Structures for Found Devices ---
enum class EncryptionType {
    OPEN,
    WEP,
    WPA_PSK,
    WPA2_PSK,
    WPA_WPA2_PSK,
    WPA2_ENTERPRISE, // Add more as needed
    UNKNOWN
};

typedef struct {
    uint8_t bssid[6];
    char ssid[MAX_SSID_LEN + 1];
    uint8_t channel;
    int8_t rssi;
    EncryptionType encryption;
    uint32_t last_seen;
    uint16_t packet_count;
    bool selected; // For UI highlighting
} AccessPointInfo;

typedef struct {
    uint8_t mac[6];
    uint8_t bssid_ap[6]; // BSSID of the AP this station is associated with (if known)
    int8_t rssi;
    uint32_t last_seen;
    uint16_t packet_count;
    bool selected; // For UI highlighting
} StationInfo;

// --- Global Variables (declared here, defined in .cpp) ---
extern AccessPointInfo ap_list[MAX_APS];
extern uint8_t ap_count;
extern StationInfo sta_list[MAX_STAS];
extern uint8_t sta_count;
extern uint8_t current_channel;
extern bool scanner_active;

// --- Function Declarations ---
void scanner_init();
void scanner_update();
void scanner_stop();
void scanner_draw(); // Function to draw scanner UI
void scanner_handle_input(ButtonPressType pressType); // Handle button presses

// Internal functions
void scanner_set_channel(uint8_t channel);
void packet_capture_callback(uint8_t *buf, uint16_t len); // SDK callback
void process_wifi_packet(uint8_t *buf, uint16_t len);
int find_ap(const uint8_t* bssid);
int find_sta(const uint8_t* mac);
void add_or_update_ap(const mac_header_t* header, const beacon_probe_fixed_params_t* params, int8_t rssi, uint8_t channel, const uint8_t* tagged_params, uint16_t tagged_params_len);
void add_or_update_sta(const uint8_t* mac, const uint8_t* bssid, int8_t rssi);
EncryptionType get_encryption_type(const uint16_t capabilities, const uint8_t* tagged_params, uint16_t tagged_params_len);
String encryption_to_string(EncryptionType enc);

// SDK function declarations
extern "C" {
    #include "user_interface.h" // Required for wifi_promiscuous_enable, etc.
    typedef void (*wifi_promiscuous_cb_t)(uint8_t *buf, uint16_t len);
    void wifi_promiscuous_enable(uint8 promiscuous);
    void wifi_set_promiscuous_rx_cb(wifi_promiscuous_cb_t cb);
    bool wifi_set_channel(uint8 channel);
    void wifi_sniffer_packet_handler(uint8_t *buff, uint16_t len);
}

#endif // SCANNER_H 