#include "ui.h"
#include "config.h"
#include "utils.h"
#include <U8g2lib.h>
#include <Wire.h>

#include "scanner.h"
#include "deauth.h"
#include "beacon.h"
#include "mitm.h"
#include "rogue_ap.h"

// --- OLED Instance ---
U8G2_SSD1306_128X64_NONAME_F_HW_I2C oled(U8G2_R0, /* reset=*/ U8X8_PIN_NONE, /* scl=*/ OLED_SCL_PIN, /* sda=*/ OLED_SDA_PIN);

// --- Global Flag for Redraw ---
bool needsRedraw = true;

// --- Menu Definition ---
const char* mainMenuItems[] = {
    "WiFi Scanner",
    "Deauth Attack",
    "Beacon Flood",
    "Evil Twin",
    "Rogue AP"
};
const int mainMenuLength = sizeof(mainMenuItems) / sizeof(mainMenuItems[0]);

// --- State Variables ---
UIState currentState = UIState::MAIN_MENU;
int currentMenuIndex = 0;

// --- Button State ---
static ButtonPressType lastPressType = ButtonPressType::NO_PRESS;
static bool buttonState = HIGH;
static bool lastButtonState = HIGH;
static unsigned long buttonDownTime = 0;
static unsigned long lastDebounceTime = 0;
static unsigned long pressStartTime = 0;
static int currentButtonState = HIGH;

// --- Battery ---
static int batteryPercentage = -1;
static unsigned long lastBatteryReadTime = 0;

// --- Initialization ---
void ui_init() {
    pinMode(BUTTON_PIN, INPUT_PULLUP);
    Serial.println("UI Initialized.");
    needsRedraw = true;
}

// --- Main Update Loop for UI ---
void ui_update() {
    unsigned long currentMillis = millis();

    // --- Button Handling ---
    currentButtonState = digitalRead(BUTTON_PIN);

    static int prevRawState = HIGH;
    if (currentButtonState != prevRawState) {
        prevRawState = currentButtonState;
    }

    ButtonPressType currentPress = ButtonPressType::NO_PRESS;

    if (currentButtonState != lastButtonState) {
        lastDebounceTime = currentMillis;
    }

    if ((currentMillis - lastDebounceTime) > DEBOUNCE_TIME) {
        if (currentButtonState != buttonState) {
            buttonState = currentButtonState;

            if (buttonState == LOW) {
                pressStartTime = currentMillis;
            } else { 
                if (pressStartTime > 0) { 
                    unsigned long pressDuration = currentMillis - pressStartTime;
                    if (pressDuration >= LONG_PRESS_TIME) {
                        currentPress = ButtonPressType::LONG_PRESS;
                    } else {
                        currentPress = ButtonPressType::SHORT_PRESS;
                    }
                    pressStartTime = 0;
                }
            }
        }
    }
    lastButtonState = currentButtonState;

    if (currentPress != ButtonPressType::NO_PRESS) {
        switch (currentState) {
            case UIState::MAIN_MENU:
                if (currentPress == ButtonPressType::SHORT_PRESS) {
                    currentMenuIndex = (currentMenuIndex + 1) % mainMenuLength;
                    needsRedraw = true;
                } else if (currentPress == ButtonPressType::LONG_PRESS) {
                    UIState nextState = (UIState)(currentMenuIndex + 1);
                    currentState = nextState;
                    currentMenuIndex = 0;
                    needsRedraw = true;
                }
                break;
            case UIState::SCANNER_MODE:
                if (currentPress == ButtonPressType::LONG_PRESS) {
                    scanner_stop();
                    currentState = UIState::MAIN_MENU;
                    currentMenuIndex = 0;
                    needsRedraw = true;
                } else {
                    scanner_handle_input(currentPress);
                }
                break;
            case UIState::DEAUTH_MODE:
                deauth_handle_input(currentPress);
                break;
            case UIState::BEACON_MODE:
                beacon_handle_input(currentPress);
                break;
            case UIState::MITM_MODE:
                mitm_handle_input(currentPress);
                break;
            case UIState::ROGUE_AP_MODE:
                rogue_ap_handle_input(currentPress);
                break;
            default:
                 if (currentPress == ButtonPressType::LONG_PRESS) {
                     scanner_stop();
                     deauth_stop();
                     beacon_stop();
                     mitm_stop();
                     rogue_ap_stop();

                     currentState = UIState::MAIN_MENU;
                     currentMenuIndex = 0;
                     needsRedraw = true;
                 }
                break;
        }
    }

    // Update Battery periodically
    if (millis() - lastBatteryReadTime > 5000 || batteryPercentage == -1) {
        float voltage = readBatteryVoltage();
        int newPercentage = estimateBatteryPercentage(voltage);
        if (newPercentage != batteryPercentage) {
            batteryPercentage = newPercentage;
            needsRedraw = true;
        }
        lastBatteryReadTime = millis();
    }

    if (needsRedraw) {
        oled.clearBuffer();
        
        const char* currentTitle = "Ozdag Attacker";
        switch (currentState) {
            case UIState::MAIN_MENU:    currentTitle = "Main Menu"; break;
            case UIState::SCANNER_MODE: currentTitle = "WiFi Scanner"; break;
            case UIState::DEAUTH_MODE:  currentTitle = "Deauth Attack"; break;
            case UIState::BEACON_MODE:  currentTitle = "Beacon Flood"; break;
            case UIState::MITM_MODE:    currentTitle = "Evil Twin"; break;
            case UIState::ROGUE_AP_MODE:currentTitle = "Rogue AP"; break;
        }
        ui_draw_header(currentTitle);

        switch (currentState) {
            case UIState::MAIN_MENU:
                ui_draw_main_menu();
                break;
            case UIState::SCANNER_MODE:
                scanner_draw();
                break;
            case UIState::DEAUTH_MODE:
                deauth_draw();
                break;
            case UIState::BEACON_MODE:
                beacon_draw();
                break;
            case UIState::MITM_MODE:
                mitm_draw();
                break;
            case UIState::ROGUE_AP_MODE:
                rogue_ap_draw();
                break;
            default:
                break;
        }
        
        ui_draw_battery(SCREEN_WIDTH - 18, 0, batteryPercentage);

        oled.sendBuffer();
        needsRedraw = false;
    }
}

// --- Drawing Functions ---
void ui_draw_header(const char* title) {
    oled.setFont(u8g2_font_6x10_tf);
    oled.drawHLine(0, 12, SCREEN_WIDTH);
    oled.drawStr(0, 10, title);
}

void ui_draw_battery(int x, int y, int percentage) {
    oled.drawFrame(x, y, 18, 9);
    oled.drawBox(x + 18, y + 2, 1, 5);

    if (percentage >= 0) {
        int width = map(percentage, 0, 100, 0, 16);
        if (width > 0) {
            oled.drawBox(x + 1, y + 1, width, 7);
        }
    } else {
        oled.drawStr(x + 6, y + 8, "?");
    }
}

void ui_draw_main_menu() {
    oled.setFont(u8g2_font_6x10_tf);

    const int itemsPerPage = 4;
    int firstItem = (currentMenuIndex / itemsPerPage) * itemsPerPage;

    for (int i = 0; i < itemsPerPage; ++i) {
        int itemIndex = firstItem + i;
        if (itemIndex >= mainMenuLength) break;

        int yPos = 24 + (i * 11);

        if (itemIndex == currentMenuIndex) {
             oled.drawStr(0, yPos, ">");
             oled.drawStr(8, yPos, mainMenuItems[itemIndex]);
        } else {
             oled.drawStr(8, yPos, mainMenuItems[itemIndex]);
        }
    }

    if (mainMenuLength > itemsPerPage) {
        if (firstItem > 0) {
             oled.drawTriangle(SCREEN_WIDTH - 6, 16, SCREEN_WIDTH - 9, 20, SCREEN_WIDTH - 3, 20);
        }
        if (firstItem + itemsPerPage < mainMenuLength) {
             oled.drawTriangle(SCREEN_WIDTH - 6, SCREEN_HEIGHT - 1, SCREEN_WIDTH - 9, SCREEN_HEIGHT - 5, SCREEN_WIDTH - 3, SCREEN_HEIGHT - 5);
        }
    }
}

// --- Getters ---
UIState ui_get_current_state() {
    return currentState;
}

int ui_get_menu_index() {
    return currentMenuIndex;
}

// --- Button Update Function ---
void button_update() {
    bool reading = digitalRead(BUTTON_PIN);

    if (reading != lastButtonState) {
        lastDebounceTime = millis();
    }

    if ((millis() - lastDebounceTime) > DEBOUNCE_TIME) {
        if (reading != buttonState) {
            buttonState = reading;

            if (buttonState == LOW) {
                buttonDownTime = millis();
            } else { 
                unsigned long pressDuration = millis() - buttonDownTime;

                if (pressDuration >= LONG_PRESS_TIME) {
                    lastPressType = ButtonPressType::LONG_PRESS;
                } else if (pressDuration >= DEBOUNCE_TIME) {
                    lastPressType = ButtonPressType::SHORT_PRESS;
                }
            }
        }
    }
    lastButtonState = reading;
}

// --- Button Get Press Function ---
ButtonPressType button_get_press() {
    ButtonPressType press = lastPressType;
    if (press != ButtonPressType::NO_PRESS) {
        lastPressType = ButtonPressType::NO_PRESS;
    }
    return press;
}

void handle_button() {
    ButtonPressType press = button_get_press();

    if (press != ButtonPressType::NO_PRESS) {
        switch (currentState) {
            case UIState::MAIN_MENU:
                if (press == ButtonPressType::SHORT_PRESS) {
                    currentMenuIndex = (currentMenuIndex + 1) % mainMenuLength;
                    needsRedraw = true;
                } else if (press == ButtonPressType::LONG_PRESS) {
                    currentState = (UIState)(currentMenuIndex + 1);
                    currentMenuIndex = 0;
                    needsRedraw = true;
                }
                break;
            case UIState::SCANNER_MODE:
                break;
            case UIState::DEAUTH_MODE:
                deauth_handle_input(press);
                break;
            case UIState::BEACON_MODE:
                beacon_handle_input(press);
                break;
            case UIState::MITM_MODE:
                mitm_handle_input(press);
                break;
            case UIState::ROGUE_AP_MODE:
                rogue_ap_handle_input(press);
                break;
            default:
                if (press == ButtonPressType::LONG_PRESS) {
                     currentState = UIState::MAIN_MENU;
                     currentMenuIndex = 0;
                     needsRedraw = true;
                }
                break;
        }
    }
}

void draw_ui() {
    oled.clearBuffer();

    switch (currentState) {
        case UIState::MAIN_MENU:
            ui_draw_main_menu();
            break;
        case UIState::SCANNER_MODE:
            scanner_draw();
            break;
        case UIState::DEAUTH_MODE:
            deauth_draw();
            break;
        case UIState::BEACON_MODE:
            beacon_draw();
            break;
        case UIState::MITM_MODE:
            mitm_draw();
            break;
        case UIState::ROGUE_AP_MODE:
            rogue_ap_draw();
            break;
        default:
            oled.setFont(u8g2_font_6x10_tf);
            oled.drawStr(0, 30, "Unknown State");
            break;
    }

    oled.sendBuffer();
} 